/*
 * Copyright (c) 2016 Open-RnD Sp. z o.o.
 * Copyright (c) 2020 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr.h>
#include <device.h>
#include <drivers/gpio.h>
#include <sys/util.h>
#include <sys/printk.h>
#include <inttypes.h>

#include <string.h>
#include <stdio.h>

#define SLEEP_TIME_MS	1

/*
 * Get reed_switch configuration from the devicetree sw0 alias. This is mandatory.
 */
#define REED_SW_NODE	DT_ALIAS(reedsw)
#if !DT_NODE_HAS_STATUS(REED_SW_NODE, okay)
#error "Unsupported board: sw0 devicetree alias is not defined"
#endif


static const struct gpio_dt_spec reed_switch = GPIO_DT_SPEC_GET_OR(REED_SW_NODE, gpios,
							      {0});
static const char *reed_label =  DT_LABEL(REED_SW_NODE);
static struct gpio_callback reed_switch_cb_data;

/*
 * The led0 devicetree alias is optional. If present, we'll use it
 * to turn on the LED whenever the reed_switch is pressed.
 */
static struct gpio_dt_spec led = GPIO_DT_SPEC_GET_OR(DT_ALIAS(led), gpios,
						     {0});

void reed_switch_pressed(const struct device *dev, struct gpio_callback *cb,
		    uint32_t pins)
{
	if(dev == reed_switch.port)
	{
		if(pins & BIT(reed_switch.pin))
		{
			int val = gpio_pin_get_dt(&reed_switch);
			printk("%s %sactivated at %" PRIu32 "\n", reed_label, val==0 ? "de-" : "", k_cycle_get_32());
			if (led.port)
			{
				gpio_pin_set_dt(&led, val);
			}
		}

	}
}

void main(void)
{
	int ret;

	if (!device_is_ready(reed_switch.port)) {
		printk("Error: reed_switch device %s is not ready\n",
		       reed_switch.port->name);
		return;
	}

	ret = gpio_pin_configure_dt(&reed_switch, GPIO_INPUT);
	if (ret != 0) {
		printk("Error %d: failed to configure %s pin %d\n",
		       ret, reed_switch.port->name, reed_switch.pin);
		return;
	}
	

	ret = gpio_pin_interrupt_configure_dt(&reed_switch,
					      GPIO_INT_EDGE_BOTH);
	if (ret != 0) {
		printk("Error %d: failed to configure interrupt on %s pin %d\n",
			ret, reed_switch.port->name, reed_switch.pin);
		return;
	}

	gpio_init_callback(&reed_switch_cb_data, reed_switch_pressed, BIT(reed_switch.pin));
	gpio_add_callback(reed_switch.port, &reed_switch_cb_data);
	printk("Set up reed_switch at %s pin %d\n", reed_switch.port->name, reed_switch.pin);

	// int has_status =DT_NODE_HAS_PROP(DT_ALIAS(led), label);
	// char *label = DT_PROP(DT_ALIAS(led), label);

	if (led.port && !device_is_ready(led.port)) {
		printk("Error %d: LED device %s is not ready; ignoring it\n",
		       ret, led.port->name);
		led.port = NULL;
	}
	if (led.port) {
		ret = gpio_pin_configure_dt(&led, GPIO_OUTPUT);
		if (ret != 0) {
			printk("Error %d: failed to configure LED device %s pin %d\n",
			       ret, led.port->name, led.pin);
			led.port = NULL;
		} else {
			printk("Set up LED at %s pin %d\n", led.port->name, led.pin);
		}
	}

	printk("Press the reed_switch\n");
	//SEGGER_RTT_printf(0,"bla\n");

	/*
	if (led.port) {
		while (1) {
			// If we have an LED, match its state to the reed_switch's.
			// int val = gpio_pin_get_dt(&reed_switch);

			// if (val >= 0) {
			// 	gpio_pin_set_dt(&led, val);
			// }
			k_msleep(SLEEP_TIME_MS);
		}
	}
	*/

static char buf[5000];
memset(buf,0,sizeof(buf));
for(int n=0; n<sizeof(buf)-10; n+=10)
{
	sprintf(&buf[n],"%10u",n);
	if(n%10==0)
		printk("%d:%s\n", n,buf);
}

printk("Ready, waiting for button...\n");

	while (1) 
	{
		gpio_pin_set_dt(&led, 0);
		k_msleep(1000);
		gpio_pin_set_dt(&led, 1);
		k_msleep(1000);
	}
}
